/*
 * File         : spmd_Stride.C
 * Author       : Guansong Zhang
 * Created      : Mon Oct 13 12:06:49 1997
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1998/09/29 19:48:11 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "spmd_Stride.h"
#include "adlib.h"
#include "iostream.h"

extern jfieldID hCPPObjID;

jfieldID offsetID=0;
jfieldID stepID=0;
jfieldID lenID=0;

Location * spmd_Location_ResolvePtr(JNIEnv *, jobject);
LocBlocksIndex * spmd_LocBlocksIndex_ResolvePtr(JNIEnv *, jobject);

#include "jctable.h"
jctable<Stride> spmd_Stride_Table;

/*ResolvePtr
 */
Stride * spmd_Stride_ResolvePtr(JNIEnv *env, jobject jthis) {
  return(spmd_Stride_Table.ObjPtr[env->GetIntField(jthis, hCPPObjID)]);
}

/*
 * Class:     spmd_Stride
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_init
  (JNIEnv * env, jclass thisClass)
{
  jclass InterfaceObject_class=env->FindClass("spmd/InterfaceObject");
  hCPPObjID=env->GetFieldID(InterfaceObject_class, "hCPPObj", "I");

  jclass Stride_class=env->FindClass("spmd/Stride");
  offsetID=env->GetFieldID(Stride_class, "offset", "I");
  stepID=env->GetFieldID(Stride_class, "step", "I");
  lenID=env->GetFieldID(Stride_class, "len", "I");

  jctable<Stride> *tempspmd_Stride_Table=new jctable<Stride>;
  spmd_Stride_Table=*tempspmd_Stride_Table;
  delete tempspmd_Stride_Table;
};

/*
 * Class:     spmd_Stride
 * Method:    clean
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_clean
  (JNIEnv *, jclass);

/*
 * Class:     spmd_Stride
 * Method:    constructor
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_constructor__
  (JNIEnv *env, jobject jthis)
{
  Stride *cppStride=new Stride();
  long pos=  spmd_Stride_Table.NewHandle(cppStride);

  //printf("Stride: %d\n",pos);

  if (spmd_Stride_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //spmd_Stride_IncRefCntr(pos);
  
  env->SetIntField(jthis, hCPPObjID, pos);

}

/*
 * Class:     spmd_Stride
 * Method:    constructor
 * Signature: (Lspmd/Stride;)V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_constructor__Lspmd_Stride_2
  (JNIEnv *, jobject, jobject);


/*
 * Class:     spmd_Stride
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_finalize
  (JNIEnv * env, jobject jthis)
{
  long thecppobj=env->GetIntField(jthis, hCPPObjID);
  long rc=spmd_Stride_Table.DecRefCntr(thecppobj);
  if (rc==0) {
    delete spmd_Stride_Table.ObjPtr[thecppobj];
    spmd_Stride_Table.ObjPtr[thecppobj]=NULL;
  }
};

/*
 * Class:     spmd_Stride
 * Method:    len
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_spmd_Stride_len
  (JNIEnv *env, jobject jthis)
{
  return(spmd_Stride_ResolvePtr(env, jthis)->len());
}

/*
 * Class:     spmd_Stride
 * Method:    disp
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_spmd_Stride_disp
  (JNIEnv * env, jobject jthis, jint sub)
{
  return(spmd_Stride_ResolvePtr(env, jthis)->disp(sub));
};


/*
 * Class:     spmd_Stride
 * Method:    disp_step
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_spmd_Stride_disp_1step
  (JNIEnv * env, jobject jthis, jint sub_stp)
{
  //  return(spmd_Stride_ResolvePtr(env, jthis)->disp_step(sub_stp));
  return printf("this function is canceled.\n");
};

/*
 * Class:     spmd_Stride
 * Method:    offset
 * Signature: (Lspmd/Location;)I
 */
JNIEXPORT jint JNICALL Java_spmd_Stride_offset
  (JNIEnv *env, jobject jthis, jobject i)
{
  return(spmd_Stride_ResolvePtr(env, jthis)->offset(*spmd_Location_ResolvePtr(env,i)));
}

/*
 * Class:     spmd_Stride
 * Method:    step
 * Signature: (Lspmd/Location;)I
 */
JNIEXPORT jint JNICALL Java_spmd_Stride_step
  (JNIEnv * env, jobject jthis, jobject i)
{
  return(spmd_Stride_ResolvePtr(env, jthis)->step(spmd_LocBlocksIndex_ResolvePtr(env,i)->sub_stp));
};

/*
 * Class:     spmd_Stride
 * Method:    updateLocBlk
 * Signature: (Lspmd/LocBlocksIndex;)V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_updateLocBlk
  (JNIEnv * env, jobject jthis, jobject jLocBlocksIndex)
{
  LocBlocksIndex * i = spmd_LocBlocksIndex_ResolvePtr(env,jLocBlocksIndex);
  Stride * std = spmd_Stride_ResolvePtr(env, jthis);

  int offset = std->offset(*i);
  int step = std->step(i->sub_stp);
  int len = std->len();

  env->SetIntField(jthis, offsetID, offset);
  env->SetIntField(jthis, stepID, step);
  env->SetIntField(jthis, lenID, len);
};

/*
 * Class:     spmd_Stride
 * Method:    show
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_spmd_Stride_show
  (JNIEnv * env, jobject jthis, jstring name)
{
  jclass Stride_class=env->FindClass("spmd/Stride");
  jfieldID sizeID=env->GetFieldID(Stride_class, "size", "I");;

  int size=env->GetIntField(jthis, sizeID);

  int length=env->GetStringUTFLength(name);

  char * cname=new char[length+1];

  const char * jnamePtr=env->GetStringUTFChars(name, 0);

  for (int i=0; i<length; i++)
    cname[i]=*(jnamePtr+i);
  cname[length]='\0';

  env->ReleaseStringUTFChars(name, jnamePtr);

  int pos =  env->GetIntField(jthis, hCPPObjID);

  AdlibPrintf("Stride %s, Handle %d, Address %x, Size %d\n", cname, pos, spmd_Stride_ResolvePtr(env, jthis), size);

  delete [] cname;

};
